import logging
from typing import Dict

import httpx

from .exceptions import ApiClientException


def compute_function_endpoint(two_dotted_function_name: str):
    uri_chunks = two_dotted_function_name.split(".")
    if len(uri_chunks) != 3:
        raise ApiClientException(
            f"Function ({two_dotted_function_name}) doesn't match standard Jelastic function (Group.Class.Function)"
        )
    grp = uri_chunks[0]
    cls = uri_chunks[1]
    fnc = uri_chunks[2]
    uri = f"{grp}/{cls}/REST/{fnc}".lower()

    return uri


class ApiClient:
    def __init__(self, api_url: str, api_token: str):
        """
        Get all needed data to connect to a Jelastic API
        """
        self.api_url = api_url
        self.api_data = {"session": api_token}
        self.logger = logging.getLogger(self.__class__.__name__)
        # the jelastic api is _synchronous_ => no timeouts
        self.client = httpx.Client(timeout=None)

    def _apicall(self, uri: str, method: str, data: dict = {}) -> Dict:
        """
        Lowest-level API call: that's the method that talks over the network to the Jelastic API
        """
        self.logger.debug(f"_apicall {method.upper()} {uri}, data:{data}")
        data.update(self.api_data)
        r = self.client.request(
            method=method, url=f"{self.api_url.strip('/')}/{uri}", data=data
        )
        if r.status_code != httpx.codes.OK:
            raise ApiClientException(
                f"{method} to {uri} failed with HTTP code {r.status_code}"
            )

        response = r.json()
        if response["result"] != 0:
            raise ApiClientException(
                f"{method} to {uri} returned non-zero result", response
            )
        self.logger.debug(f"response : {response}")
        return response

    def execute(self, function: str, **kwargs) -> Dict:
        """
        Direct API call, converting function paths into URLs; allows:
            api_client.execute('Environment.Control.GetEnvs')
        """
        self.logger.info(f"{function}({kwargs})")
        uri = compute_function_endpoint(function)

        return self._apicall(uri=uri, method="post", data=kwargs)



from typing import NamedTuple, List


class DockerSettings(NamedTuple):
    image: str
    nodeGroup: str = None


class EnvNode(NamedTuple):
    docker: DockerSettings = None
    count: int = 1
    displayName: str = None
    extip: bool = False
    fixedCloudlets: int = None
    flexibleCloudlets: int = None
    nodeType: str = None


EnvNodes = List[EnvNode]



from .core import ApiClient, BaseClient, who_am_i


class JpsClient(BaseClient):
    jelastic_group = "marketplace"

    jelastic_class = "jps"

    def __init__(self, api_client: ApiClient):
        super().__init__(api_client)

    def install(self, filename: str, env_name: str) -> None:
        with open(filename) as file:
            manifest_content = file.read()

            self._execute(
                who_am_i(),
                jps=manifest_content,
                envName=env_name
            )

    def get_engine_version(self) -> str:
        response = self._execute(
            who_am_i()
        )

        return response["version"]



from __future__ import annotations

from jelastic_client import FileClient


def test_file_client_read_file_from_environment(
        file_client: FileClient,
        alpine_with_file: tuple[str, str],
        expected_file_content_in_alpine_with_file: str):
    # Arrange
    env_name = alpine_with_file[0]
    path_to_file = alpine_with_file[1]

    # Act
    actual_content = file_client.read(
        env_name, path_to_file, node_type="docker")

    # Assert
    assert expected_file_content_in_alpine_with_file.replace(
        "\r", "") == actual_content.replace("\r", "")



import random
import string

from jelastic_client import ControlClient


def create_random_env_name(commit_sha: str, worker_id: str) -> str:
    env_id = "".join(random.choice(string.digits) for _ in range(7))
    return "-".join([commit_sha, worker_id, env_id])


def get_new_random_env_name(control_client: ControlClient, commit_sha: str, worker_id: str) -> str:
    env_name = create_random_env_name(commit_sha, worker_id)
    while control_client.env_exists(env_name):
        env_name = create_random_env_name(commit_sha, worker_id)
    return env_name



from typing import NamedTuple


class EnvSettings(NamedTuple):
    displayName: str = None
    engine: str = None
    ishaenabled: bool = False
    region: str = None
    shortdomain: str = None
    sslstate: bool = False



from .control_client import ControlClient
from .env_node import EnvNode, DockerSettings
from .env_settings import EnvSettings
from .factory import JelasticClientFactory
from .file_client import FileClient
from .jps_client import JpsClient



import inspect

from .api_client import ApiClient
from .base_client import BaseClient
from .exceptions import ApiClientException, JelasticClientException


def who_am_i() -> str:
    function_name_in_snake_case = inspect.stack()[1][3]
    return "".join(function_name_in_snake_case.split('_'))
